(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: code.mli,v 1.10 2007/04/26 15:34:47 weis Exp $ *)

(** The Caml code we generate. *)

type lident = string;;
(** Lowercase ident. *)
type uident = string;;
(** Uppercase ident. *)
type iident = string;;
(** Infix ident. *)
type ident = string;;
(** Any kind of ident. *)

type exp =
   | Id of ident
   | Match of exp * clause list
   | Try of exp * clause list
   | Constr of uident * exp list
   | App of exp * exp list
   | AppI of exp * iident * exp
   | If of exp * exp * exp
   | Let of lident * lident list * exp * exp
   | Letrec of lident * lident list * exp * exp
   | Record of (lident * exp) list
   | Pair of exp * exp
   | Ocaml_pattern of Parsetree.pattern
   | Ocaml_expression of Parsetree.expression
   | Function of clause list
   | Comment of string

and clause =
  | Clause of exp * exp
  | Clause_when of exp * exp * exp;;

type structure_item =
   (** Defines a set of possibly recursive functional values *)
   | Pstr_value of Asttypes.rec_flag * (lident * lident list * exp) list
   (** Defines one primitive *)
   | Pstr_primitive of string * Parsetree.value_description
   (** Defines one type *)
   | Pstr_type of (lident * Parsetree.type_declaration) list;;

type structure = structure_item list;;

val topletrec : (lident * lident list * exp) list -> structure_item;;

(** Globally predefined identifiers. *)

val underscore : exp;;
val x : exp;;
val y : exp;;
val z : exp;;
val t : exp;;
val u : exp;;

val genr_args : string -> int -> lident list;;
(** [genr_args root n] returns the list of strings [root1; ...; rootn]. *)

val exps_of_idents : lident list -> exp list;;
val idents_of_exps : exp list -> lident list;;

val genr_fun_args : int -> lident * lident list;;
(** [genr_fun_args n], for [n >= 1], returns the (non empty) list of strings
    ("x1", ["x2"; ... "xn"]). *) 

val genr_idargs : lident -> int -> exp list;;
(** Same as [genr_fun_args] but returns [Id] expressions. *)
val genr_numbered_idargs : lident -> int -> (int * exp) list;;
(** Same as [genr_fun_args] but returns [Id] expressions. *)

(** Application macros. *)

val app1 : lident -> exp -> exp;;
val app2 : lident -> exp -> exp -> exp;;
val app3 : lident -> exp -> exp -> exp -> exp;;
(** Apply an ident to its argument(s). *)

val appi : exp -> iident -> exp -> exp;;
(** Apply an infix ident to its arguments. *)

val constr1 : uident -> exp -> exp;;
val constr2 : uident -> exp -> exp -> exp;;
(** Apply a constructor to its argument(s). *)

(** Other useful auxilliaries. *)

val let0 : lident -> exp -> exp -> exp;;
val let1 : lident -> lident -> exp -> exp -> exp;;
val pair : exp -> exp -> exp;;
val raise_error : string -> exp;;

(** Printing the code. *)

val structure : Buffer.t -> structure -> unit;;
val structure_item : Buffer.t -> structure_item -> unit;;
