(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr.ml,v 1.13 2007/04/25 15:04:50 weis Exp $ *)

(* The main generation module for construction functions. *)

open Spec;;
open Code;;

open Genr_base;;

let genr_fun_0 fdef =
  [construction_function_name fdef.fd_uname, [], Constr (fdef.fd_uname, [])];;

let genr_fun_n fdef =
  let args = Code.genr_args "x" fdef.fd_arity in
  let body = constr fdef.fd_uname (Code.exps_of_idents args) in
  [construction_function_name fdef.fd_uname, args, body];;

let genr_cdef fdef =
  match fdef.fd_arity with
  | 0 -> genr_fun_0 fdef
  | 1 -> Genr_unary.genr_fun_1 fdef
  | 2 -> Genr_binary.genr_fun_2 fdef
  | _ -> genr_fun_n fdef;;

let genr_record tname fields =
  let genr_field fn = (fn, Id fn) in
  [tname, fields, Record (List.map genr_field fields)];;

let genr_abbrev tname ty rels = failwith "Not yet implemented";;

let genr_tdef tdef =
  match tdef.td_desc with
  | TypeRecord fields -> genr_record tdef.td_name fields
  | TypeSum cdefs ->
      List.flatten (List.map genr_cdef (Complete.genr_fdefs cdefs))
  | TypeAbbrev (ty, rels) -> genr_abbrev tdef.td_name ty rels;;

let topexp_of_tdefs tdefs =
  topletrec (List.flatten (List.map genr_tdef tdefs)) ::
  List.map (fun tdef -> Genr_base.genr_eq tdef.td_name) tdefs;;
